unit wind2;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,wind,globals,misc,crt,comms,hardc,scrn;
function tohex(d:real;n:byte):string;
function tobin(d:real;n:byte):string;
function flipstr(s:string;n:byte):string;
function getoffset(dformat,iptype:byte):real;
procedure rxconfigval;
procedure chconfigval;
procedure diagval;
procedure exinput(var exip:real);
procedure outfsd(var opfsd:real);
procedure infsd(var ipfsd:real);
procedure inzero(var ipzero:real);
procedure calcgain(inzero,infsd,outfsd:real);
procedure calcvals(ipzero,ipfsd,opfsd,exin,exout:real);
procedure confirmram;
procedure dfault;
procedure copyrx;
procedure gocopy(source,dest:opt; var dload:byte);
procedure copydown(rx,tx:byte);
procedure signoff;
procedure inputype;
procedure readchecksum;

implementation
function tohex(d:real;n:byte):string;
{ d=decimal number to convert, n=number of hex digits required }
{ returns the hex number as a string of n characters }

var dv:real;
    h:array[1..8] of char;
    i:integer;
    temp:byte;

begin
dv:=1;
for i:=2 to n do dv:=dv*16;
for i:=1 to n do
  begin
    temp:=trunc(d/dv);
    d:=d-temp*dv;
    dv:=dv/16;
    case temp of
         0..9:h[i]:=chr(48+temp);
         10..15:h[i]:=chr(55+temp);
         else h[i]:='?';
         end;
  end; {for i}

  tohex:=copy(h,1,n);

end; {tohex}
{**************************************************************************}
function tobin(d:real;n:byte):string;
{ d=decimal number to convert, n=number of binary digits required }
{ returns the binary number as a string of n characters }

var dv:real;
    h:array[1..32] of char;
    i:integer;
    temp:byte;

begin
for i:=1 to 32 do h[i]:='.';
dv:=1;
for i:=2 to n do dv:=dv*2;
for i:=1 to n do
  begin
    temp:=trunc(d/dv);
    d:=d-temp*dv;
    dv:=dv/2;
    case temp of
         0..1:h[i]:=chr(48+temp);
         else h[i]:='?';
         end;
  end; {for i}

  tobin:=copy(h,1,n);

end; {tohex}

{**************************************************************************}
function flipstr(s:string;n:byte):string;
{ gives the mirror image of the string, the length of the string }
{ defined by n}
var h:array[1..32] of char;
    i:byte;
begin
for i:= 1 to n do
         h[i]:=s[n-i+1];

flipstr:=copy(h,1,n);
end;
{**************************************************************************}
function getoffset(dformat,iptype:byte):real;

begin
  if unipolar(dformat) then
     begin
        case iptype of
          0:getoffset:=100;    {mV input}
          13,15:getoffset:=0;  {RTD, ohmic output or switch}
          else getoffset:=500; {temperature output}
          end; {case iptype}
     end
     else
          { bi-polar format so no offset required }
         getoffset:=0;
end; {getoffset}
{*********************************************************************}
procedure rxconfigval;

{ receiver configuration screen - display values }


var
   i,temptype,t:byte;
   a:boolean;
   tempst:string[20];

begin

rtd:=false;
with receiver[recno]^ do
     begin {with_1}
{     if txno > no_of_tx then txno:=1;}
     with transmitter[txno] do
          begin {with_2}
	  rtd:=IsRtd(recno,txno);
	  with channel[chno] do
          begin {with_3}
           disp[1,1]:=' '; {reset receiver}
	   disp[1,2]:=tag_field; {tag name}

           case data_format of
                0..1:disp[1,3]:=' '+dformatstg[data_format+1];
                4..14:disp[1,3]:=' '+dformatstg[data_format-1];
                16..18:disp[1,3]:=' '+dformatstg[data_format-2];
                else disp[1,3]:='   unknown   ';
             end;

	   disp[1,4]:=' '; {reset scales}
	   disp[1,5]:='      '+stg(no_of_tx,1);

	   case units of
		1:disp[1,6]:='     '+chr(248)+'C';
		2:disp[1,6]:='     '+chr(248)+'F';
		3:disp[1,6]:='      K';
                else disp[1,6]:='error';
                end;

	   if txtype = 1
	      then
		  disp[1,7]:='     '+stg(10*line_rej+50,2)+'Hz'
	      else
		  disp[1,7]:=message[98];

           if (check_mode=0) then disp[1,8]:='     AUTO'
                             else disp[1,8]:='    MANUAL';

           if receiver[recno]^.cksum_base=0
           then
             str(checksum:9,disp[1,9])
           else
             disp[1,9]:='     '+tohex(checksum,4)+  '  hex';

           if add_mode then
                 disp[2,1]:='   Software'
              else
                 disp[2,1]:='   Hardware';

	   disp[2,2]:='     '+stg(mod_addressA,2);
	   disp[2,3]:='     '+stg(mod_addressB,2);

           if comm_mode then
                 disp[2,4]:='   Software'
              else
                 disp[2,4]:='   Hardware';
           case mod_baud of
                1:disp[2,5]:='     300';
                2:disp[2,5]:='     600';
                3:disp[2,5]:='    1200';
                4:disp[2,5]:='    2400';
                5:disp[2,5]:='    4800';
                6:disp[2,5]:='    9600';
                7:disp[2,5]:='    19200';
                8:disp[2,5]:='    38400';
                else disp[2,5]:='    ???';
           end;

           disp[2,6]:='      '+stg(mod_stop,1); {stop bits}

           if (receiver[recno]^.mod_dbits=1) then disp[2,7]:='      8'
                                             else disp[2,7]:='      7';


           case receiver[recno]^.mod_parity of
                0:disp[2,8]:='    none';
                1:disp[2,8]:='     odd';
                2:disp[2,8]:='    even';
                end;
           case receiver[recno]^.mod_mode of
                0:disp[2,9]:='    ASCII';
                1:disp[2,9]:='     RTU';
                2:disp[2,9]:='   Unknown';
                end;
           disp[2,10]:=' ';
          end; {with_3}

         end; {with_2}

      end; {with_1}


end;


{***************************************************************************}
procedure  chconfigval;

{ channel and transmitter configuration - display values }


var
   i,temptype,t:byte;
   tempst:string[10];
   a:boolean;

begin

rtd:=false;
with receiver[recno]^ do
     begin {with_1}
     if txno>no_of_tx then txno:=1;
     with transmitter[txno] do
          begin {with_2}

          { if 831 then check if working in RTD mode }
          if txtype=1 then  rtd:=IsRtd(recno,txno)
                      else rtd:=false;

          with channel[chno] do
          begin {with_3}

          disp[1,1]:='      '+stg(txno,1);
          disp[1,3]:='     '+stg(chno,2);
          if txtype=1
                           then
                               disp[1,2]:='     831'
			   else
                               disp[1,2]:='     832';

	  disp[1,4]:=iptypestring(txtype,input_type);

          if (txtype=1) then
	   begin
               disp[1,5]:=message[thc_drive+27];

               if input_type=15 then   { old switch setting - now illegal }
                        begin
                              disp[1,5]:=message[98];
                              disp[1,6]:=message[98];
                              disp[1,7]:=message[98];
                        end;

               if (not(rtd and (chno=16)))
                   then
                       begin
                            { high and low alarms }
			    disp[1,6]:=fstr(high_alarm,3);
			    disp[1,7]:=fstr(low_alarm,3);
		       end
		   else
                       begin
                             {channel 16 - RTD sense }
                             disp[1,4]:=message[97]; {RTD Sense}
                             disp[1,6]:='    Fixed      ';
                             disp[1,7]:='    Fixed      ';
                       end;

            end; {if txtype=1}


            if (txtype=2)
             then begin

               { display safety drive, not applicable for 832 }

               disp[1,5]:=message[98];

               { display high and low alarm settings }

               disp[1,6]:=fstr(high_alarm,3);
               disp[1,7]:=fstr(low_alarm,3);
            end; {txtype=2}


            {display scalings}

	    disp[2,1]:=fstr(scale_ipzero,txtype-1);
            str(scale_opzero:10:4,disp[2,2]);
            str(scale_gain:10:4,disp[2,3]);
            disp[2,4]:='';  { calc scalings }

	  end; {with_3}

         end; {with_2}

      end; {with_1}


end;


{************************************************************************}
procedure diagval;
begin

with receiver[recno]^ do
  begin
     disp[1,2]:='1......8';
     disp[1,3]:='   '+flipstr(tobin(hcaswitch,8),8);
     disp[1,4]:='   '+flipstr(tobin(ipswitch,8),8);
     if receiver[recno]^.cksum_base=0
           then
            begin
             str(checksum:9,disp[1,5]);
             str(ramchksum:9,disp[1,6]);
            end
           else
            begin
             disp[1,5]:='     '+tohex(checksum,4)+  '  hex';
             disp[1,6]:='     '+tohex(ramchksum,4)+  '  hex';
           end;
     disp[2,1]:='';
     disp[2,2]:='   '+tobin(status1,8); {status 1}
     disp[2,3]:='   '+tobin(status2,8); {status 2}
     if (status2 and 1)<>0 then
            disp[2,4]:=' YES ' {ram corruption}
          else
            disp[2,4]:=' NO ';

  end; {with receiver}
end;

{**************************************************************************}
procedure outfsd(var opfsd:real);

var
   old:real;

begin
txt[0]:='   '+screen[scno].cell[xcur,ycur].title;

with receiver[recno]^.transmitter[txno].channel[chno] do
   begin
    old:=opfsd;
    opfsd:=getnum(old,0);
   if (unipolar(receiver[recno]^.data_format) and (opfsd < 0)) then
      begin
      opfsd:=old;
      saynonegatives;
      end;
   end;
end;

{**************************************************************************}
procedure infsd(var ipfsd:real);

var
   old:real;

begin
txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
old:=ipfsd;
ipfsd:=getnum(old,0);
end;
{**************************************************************************}
procedure inzero(var ipzero:real);

var
   old:real;

begin

txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
with receiver[recno]^.transmitter[txno].channel[chno] do
begin
  old:=ipzero;
  ipzero:=getnum(old,0);
  scale_ipzero:=ipzero+getoffset(receiver[recno]^.data_format,input_type);
end; {with receiver..}
end;

{**************************************************************************}
procedure exinput(var exip:real);

var
   old:real;

begin
txt[0]:='   '+screen[scno].cell[xcur,ycur].title;

with receiver[recno]^.transmitter[txno].channel[chno] do
   begin
    old:=exip;
    exip:=getnum(old,0);
   end;
end;

{**********************************************************************}
procedure calcgain(inzero,infsd,outfsd:real);

begin
with receiver[recno]^.transmitter[txno].channel[chno] do
begin
if (infsd-inzero)<>0 then
scale_gain:=(outfsd-scale_opzero)/(infsd-inzero);
end; {with receiver...}

end;

{**********************************************************************}
procedure calcvals(ipzero,ipfsd,opfsd,exin,exout:real);

var ttype:byte;

begin

ttype:=receiver[recno]^.transmitter[txno].txtype;

with receiver[recno]^.transmitter[txno].channel[chno] do
 begin

    disp[1,1]:=' ';
    disp[1,2]:=fstr(ipzero,ttype-1);
    disp[1,3]:=fstr(ipfsd,ttype-1);
    if ((receiver[recno]^.data_format=0) or (receiver[recno]^.data_format=1))
       then
       begin {IEEE}
            str(scale_opzero:10:4,disp[1,4]);
            str(opfsd:10:4,disp[1,5]);
       end
       else {binary or BCD}
       begin
            str(scale_opzero:10:0,disp[1,4]);
            str(opfsd:10:0,disp[1,5]);
       end;
    disp[1,6]:=' ';
    str(exin:10:4,disp[1,7]);
    if ((receiver[recno]^.data_format=0) or (receiver[recno]^.data_format=1))
       then  {IEEE}
            str(exout:10:4,disp[1,8])
       else {binary or BCD}
            str(exout:10:0,disp[1,8]);
    disp[2,1]:='';
    disp[2,2]:='      '+stg(txno,1);
    disp[2,3]:='     '+stg(chno,2);
    disp[2,4]:=fstr(scale_ipzero,ttype-1);
    str(scale_opzero:10:4,disp[2,5]);
    str(scale_gain:10:4,disp[2,6]);
    disp[2,7]:=fstr(getoffset(receiver[recno]^.data_format,input_type),ttype-1);
 end;

end;

{************************************************************************}
procedure confirmram;

var temp:byte;

begin
msgerror:=false;
escape:=false;

txt[0]:='    Clear Flag?   ';
txt[1]:='        NO        ';
txt[2]:='        YES       ';
drawwindow(30,15,2);
temp:=selection(2,1,1);
if temp=2 then
   begin
        if online then
           begin
             wrtworking;
	     send_confirm(recno);
             read_checksum(recno);
	     gotoxy(1,25);
             write('            ');
             if msgerror then
                begin
                  wrterror;
                end
           end { if online }
         else
             receiver[recno]^.status2:=receiver[recno]^.status2 and $fe;

   end; {temp=2}

end;
{************************************************************************}


procedure dfault;

var
   rectype:byte;

begin
msgerror:=false;
escape:=false;

recno:=0;
{backgnd;}
{fillscn(xcur,ycur);}
txt[0]:='     Tx Type      ';
txt[1]:='       831        ';
txt[2]:='       832        ';
drawwindow(30,15,2);
rectype:=selection(2,1,20);
if not(escape) then
   begin
    window(1,1,80,25);
    textbackground(black);
    gotoxy(1,25);
    write(message[53]);
    reset_rx(recno,rectype);
    gotoxy(1,25);
    write('            ');
    if msgerror then
       begin
	 window(1,1,80,25);
	 gotoxy(5,23);
	 textbackground(errbar);
	 write(errmsg);
	 write(^g);
	 delay(2000);
       end; {if msgerror}
   end; {if not(escape)}

end;
{************************************************************************}

procedure copyrx;

var
  source,dest:opt;   {type opt defined in WINDV24}
  option,oldx,oldy,oldrx,oldtx,oldchn,maxnotx,dload :byte;
  i:integer;

{-------------------------------------------------------------------}
procedure displayopt;
{ display the currently selected options }
begin
   backgnd(0);
   gotoxy(1,25);
   textbackground(black);
   textcolor(white);
   write(message[52]); {Esc to Abort}
   disp[1,1]:='';
   if source.transmitter=$ff then disp[1,2]:='  '+'ALL'
                              else disp[1,2]:='  '+stg(source.transmitter,2);
   if source.channel=$ff then     disp[1,3]:='  '+'ALL'
                         else     disp[1,3]:='  '+stg(source.channel,2);

   disp[2,1]:='';
   if dest.transmitter=$ff   then disp[2,2]:='  '+'ALL'
                             else disp[2,2]:='  '+stg(dest.transmitter,2);
   if dest.channel=$ff    then disp[2,3]:='  '+'ALL'
                          else disp[2,3]:='  '+stg(dest.channel,2);
   disp[2,5]:='';
end; {displayopt}
{-------------------------------------------------------------------}
procedure selstx;
{ select the source transmitter }
var i:byte;
begin
 txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
 oldtx:=source.transmitter;
 if source.receiver=$ff
    then maxnotx:=4
    else maxnotx:= receiver[source.receiver]^.no_of_tx;
 for i:=1 to maxnotx do
     txt[i]:=stg(i,10)+'           ';

 {if selected receiver is configured for more than
 one transmitter then display ALL option}
 if maxnotx > 1 then
    begin
      maxnotx:=maxnotx+1;
      txt[maxnotx]:='        ALL       ';
    end;

 {oldtx selects the previously selected option}
 {this statement ensures oldtx is not greater than
  current menu length!}
 if oldtx=$ff
   then oldtx:=maxnotx
   else if oldtx>maxnotx then oldtx:=1;

 drawwindow(30,15,maxnotx);

 i:=selection(maxnotx,oldtx,oldtx);
   {oldtx is both the initial position for the selection cursor
    and the returned option if ESC is pressed}

 if (i=maxnotx) and (i>1)
 then begin {ALL transmitters selected}
        source.transmitter:=$ff;
        dest.transmitter:=$ff; {copy from ALL must be TO ALL}
      end
 else source.transmitter:=i;  {single transmitter selected}

end; {selstx}
{-------------------------------------------------------------------}
procedure seldtx;
{ select destination transmitter }
var i:byte;
begin
  if source.transmitter=$ff
  then
   begin
     {copy from ALL transmitters must be TO ALL transmitters}
     dest.transmitter:=$ff;
     txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
     txt[1]:='        ALL       ';
     drawwindow(30,15,1);
     i:=selection(1,1,1);
   end
  else
    begin
     txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
     oldtx:=dest.transmitter;
     if dest.receiver=$ff
        then maxnotx:=4
        else maxnotx:= receiver[dest.receiver]^.no_of_tx;
     for i:=1 to maxnotx do
         txt[i]:=stg(i,10)+'           ';
     maxnotx:=maxnotx+1;
     txt[maxnotx]:='        ALL       ';

     if oldtx=$ff
       then oldtx:=maxnotx
       else if oldtx>maxnotx then oldtx:=1;
     drawwindow(30,15,maxnotx);
     i:=selection(maxnotx,oldtx,oldtx);
     if (i=maxnotx) and (i>1) then dest.transmitter:=$ff
                              else dest.transmitter:=i;
  end; {else}
end;  {seldtx}

{-------------------------------------------------------------------}

begin {copyrx}

{ set source to the current selected receiver/transmitter/channel }
source.receiver:=recno;
source.transmitter:=txno;
source.channel:=chno;
dest.receiver:=recno;
dest.transmitter:=txno;
dest.channel:=chno;
oldx:=xcur;
oldy:=ycur;
xcur:=1;
ycur:=2;
maxnotx:=1;
oldchn:=1;
oldtx:=1;
oldrx:=0;

repeat
  begin {repeat}
   displayopt;
   option:=sel(7);
   case option of
          $11:;  {null-heading}
          $12: selstx;  {select source transmitter(s)}

          $13:  {select source channel(s)}
              begin
                txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
                for i:=1 to 16 do
                     txt[i]:=stg(i,10)+'         ';
                txt[17]:='       ALL        ';
                drawwindow(30,3,17);
                if dest.channel=$ff then oldchn:=17
                                    else oldchn:=source.channel;

                i:=selection(17,oldchn,oldchn);
                case i of
                  1..16:source.channel:=i;
                  17: begin  {ALL channels selected}
                        source.channel:=$ff;
                        dest.channel:=$ff; {copy from ALL must be TO ALL}
                      end;
                  end; {case i of}
              end; {case $14}


          $21:;  {null-heading}
          $22: seldtx; {select destination transmitter(s)}

          $23:  {select destination channels(s)}
             begin
             if source.channel=$ff
              then
               begin
                 {copy from ALL channels must be TO ALL channels}
                 dest.channel:=$ff;
                 txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
                 txt[1]:='        ALL       ';
                 drawwindow(30,15,1);
                 i:=selection(1,1,1);
               end
              else
               begin
                txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
                for i:=1 to 16 do
                     txt[i]:=stg(i,10)+'         ';
                txt[17]:='       ALL        ';
                drawwindow(30,3,17);
                if dest.channel=$ff then oldchn:=17
                                    else oldchn:=dest.channel;

                i:=selection(17,oldchn,oldchn);
                case i of
                  1..16:dest.channel:=i;
                  17:   dest.channel:=$ff;
                  end;
               end; {else}
              end; {case $24}
	  $24: {go copy}
	       begin
                  gocopy(source,dest,dload); {go,action selected copy}
                  if (dload=1) and online
                    then begin
			 backgnd(0);
                         fillscn(xcur,ycur);
                         txt[0]:='    Download ?    ';
                         txt[1]:='       Yes        ';
                         txt[2]:='       No         ';
                         drawwindow(30,10,2);
                         dload:=selection(2,2,2); { default/esc option is NO }
                         if dload=1 then
                            copydown(dest.receiver,dest.transmitter);
                         end
                     else begin
                          txt[0]:='  Copy Function   ';
                          txt[1]:='    COMPLETED     ';
                          txt[2]:='                  ';
                          drawwindow(30,10,2);
                          delay(2000);
                          end;
                  xcur:=2;
                  ycur:=3;
                  end;
          end; {case}
end {repeat}
until option=$FF;
xcur:=oldx;
ycur:=oldy;
end;
{************************************************************************}

procedure gocopy(source,dest:opt; var dload:byte);
{copies configuration between selected channels/receivers}
{last modified 1/1/92 aps call to set_analogue changed}

var
 i,drx,dtx,dch,srx,stx,sch:byte;
 a:boolean;

{-------------------------------------------------------------------}
procedure copy_channel(var dload:byte);
{ copies from srx.stx.sch to drx.dtx.dch }
begin
  receiver[drx]^.transmitter[dtx].channel[dch]
		     := receiver[srx]^.transmitter[stx].channel[sch];
  if online and (dload=0)
       then
	  begin
	  { download channel configuration }
	  msgerror:=false;
	  with receiver[drx]^.transmitter[dtx].channel[dch] do
	    begin
	      gotoxy(1,25);
	      if not msgerror then
		begin
		write(' Downloading Rx '+stg(drx,2)+' Tx '+stg(dtx,2)+' Ch '+stg(dch,2));
		set_inptype(drx,dtx,dch);
		set_scales(drx,dtx,dch);
                set_alarms(drx,dtx,dch);
                end;
            end; {with}
            if msgerror then
              begin
                 window(1,1,80,25);
                 gotoxy(5,23);
                 textbackground(errbar);
                 write(errmsg);
                 gotoxy(80,25);
                 write(^g);
                 delay(2000);
                 gotoxy(5,23);
                 textbackground(back);
                 write('                        ');
                 end;
            textbackground(black);
            gotoxy(1,25);
            write('                                   ');

          end; {download channel}
end; {copy_channel}


{-------------------------------------------------------------------}
procedure copy_transmitter(source,dest:opt;var dload:byte);
{ copies from srx.stx to drx.dtx }
var i:byte;

begin
if dest.channel=$ff
    then
       begin
       {copy TO all channels}
       dload:=1; {requires explicit download}
       if source.channel=$ff {copy FROM all channels}
          then  {copy complete transmitters}
            receiver[drx]^.transmitter[dtx]:=receiver[srx]^.transmitter[stx]
          else { copy single channels to all channels}
             begin
               sch:=source.channel;
               for i:= 1 to 16 do
                 begin
                   dch:=i;
                   copy_channel(dload);
                 end ;
             end;
       end
    else
       {copy from single channel to single channel}
       begin
          sch:=source.channel;
          dch:=dest.channel;
          copy_channel(dload);
       end;

end; {copy_transmitter}


{-------------------------------------------------------------------}
procedure copy_receiver(source,dest:opt;var dload:byte);
var i:byte;

begin
if dest.transmitter=$ff
    then
       {copy TO all transmitters}
       if source.transmitter=$ff
          then
             begin
             {copy FROM all transmitters}
              dload:=1; { requires explicit download }
              for i:=1 to 4 do
                 begin
                   dtx:=i;
                   stx:=dtx;
                   copy_transmitter(source,dest,dload);
                 end;
             end

          else { copy single transmitter to all transmitters}
            begin
              dload:=1; { requires explicit download }
              stx:=source.transmitter;
              for i:= 1 to 4 do
                 begin
                  dtx:=i;
                  copy_transmitter(source,dest,dload);
                 end;
            end
    else
       {copy from single transmitter to single transmitter}
       begin
         stx:=source.transmitter;
         dtx:=dest.transmitter;
         copy_transmitter(source,dest,dload);
       end;

end; {copy_receiver}


{-------------------------------------------------------------------}

begin {gocopy}
srx:=source.receiver;
dload:=0;
{copy single receiver}
  begin
    drx:=dest.receiver;
    copy_receiver(source,dest,dload);
  end;


end; {gocopy}


{**************************************************************************}
procedure copydown(rx,tx:byte);

var
   i:integer;

begin

case rx of
     0..15:dump(rx,tx);
     $ff:for i:=0 to 15 do
            dump(i,tx);
     end;
if msgerror then
      begin
      wrterror;
      gotoxy(5,23);
      textbackground(back);
      write('                        ');
      end; {if msgerror}
textbackground(black);
gotoxy(1,25);
write('                                   ');

end;

{************************************************************************}


procedure signoff;

begin
signoffmess;
window(1,1,80,25);
textbackground(black);
gotoxy(1,25);
write(message[53]);
if not escape then
   begin
      msgerror:=false;
      send_confirm(recno);
      send_signoff(recno);
      gotoxy(1,25);
      write('            ');
      if msgerror then
         begin
           window(1,1,80,25);
           gotoxy(5,23);
           textbackground(errbar);
           write(errmsg);
           write(^g);
           delay(2000);
         end; {if msgerror}
   end; {if not escape}
end; {signoff}


{************************************************************************}
procedure inputype;

var
   i:integer;
   old:byte;
   temptype:byte;
   done:boolean;

begin
txt[0]:='   '+screen[scno].cell[xcur,ycur].title;
if not (rtd and (chno=16)) then
 with receiver[recno]^.transmitter[txno].channel[chno] do
     begin {do}
     old:=input_type;
     if old=15 then old:=0;   {switch setting no longer allowed}
     temptype:=old;

     if receiver[recno]^.transmitter[txno].txtype= 1 then
        begin
        done:=false;
        while not done do
        begin
          if temptype in[0..14] then
               begin
                  for i:=1 to 16 do
                            txt[i]:='  '+iptype831a[i];
                  if chno=16
                        then
                             begin
                                 txt[14]:=txt[16];
                                 backgnd(0);
                                 drawwindow(30,5,14);
                                 temptype:=selection(14,temptype+1,101)-1;
                             end
                        else
                             begin
		                 backgnd(0);
                                 drawwindow(30,5,16);
                                 temptype:=selection(16,temptype+1,101)-1;
                             end;

                  case temptype of
                    0..12,14:
                          begin
                             done:=true;
                             input_type:=temptype;
                          end;
                    13:if chno=16 then temptype:=16
                               else
                               begin
                                 done:=true;
                                 input_type:=temptype;
                               end;
                    15:temptype:=16; {more}
                    else done:=true; {escape}
                    end; {case}
               end
             else
               begin
                  for i:=1 to 16 do
                            txt[i]:='  '+iptype831b[i];
		  backgnd(0);
                  drawwindow(30,5,13);
                  temptype:=selection(13,temptype-15,101)+15;
                  case temptype of
                    16..27:
                           begin
                                done:=true;
                                input_type:=temptype;
                           end;
                    28:temptype:=0; {more}
                    else done:=true; {escape}
                    end; {case}
               end; {if temptype - else}

           end; {while not done}
        end {txtype=1}
     else {txtype=2}
        begin
        for i:=1 to 3 do
            txt[i]:='  '+iptype832[i];

        drawwindow(30,12,3);

        case input_type of
             0:temptype:=1;
             13:temptype:=2;
             15:temptype:=3;
             else temptype:=1;
             end;
        temptype:=selection(3,temptype,101);
        case temptype of
             1:input_type:=0;
             2:input_type:=13;
             3:input_type:=15;
             else done:=true; {escape}
             end;
        end; {txtype=2}

     if online and not(escape) then
        begin
        wrtworking;
        set_inptype(recno,txno,chno);
        gotoxy(1,25);
        write('            ');
        if msgerror then
           begin
           input_type:=old;
           wrterror;
           end; {if msgerror}
        end; {if online}
     end {do}
else
    begin
    window(1,1,80,25);
    gotoxy(5,23);
    textbackground(errbar);
    write(message[98]);
    write(^g);
    delay(2000);
    end;

end;
{************************************************************************}
procedure readchecksum;

var
   old1,old2:longint;

begin
{
with receiver[recno]^ do
     begin
       old1:=checksum;
       old2:=ramchksum;
       wrtworking;
       read_checksum(recno);
       gotoxy(1,25);
       write('            ');
       if msgerror then
	  begin
                wrterror;
		checksum:=old1;
                ramchksum:=old2;
          end; {then..}

     end;
end;}

end;

end.

